@@ -124,7 +124,10 @@ wl_goal(WorkList, Goal, Skeleton) :-
 
 trie_goal(ATrie, Goal, Skeleton) :-
     '$tbl_table_status'(ATrie, _Status, M:Variant, Skeleton),
-    M:'$table_mode'(Goal0, Variant, _Moded),
+    (   M:'$table_mode'(Goal0, Variant, _Moded)
+    ->  true
+    ;   Goal0 = Variant                 % dynamic IDG nodes
+    ),
     unqualify_goal(M:Goal0, user, Goal).
 
 delay_goals(List, Goal) :-
@@ -139,7 +142,13 @@ user_goal(Goal, UGoal) :-
 user:portray(ATrie) :-
     '$is_answer_trie'(ATrie, _),
     trie_goal(ATrie, Goal, _Skeleton),
-    format('~q for ~p', [ATrie, Goal]).
+    (   '$idg_falsecount'(ATrie, FalseCount)
+    ->  (   '$idg_forced'(ATrie)
+        ->  format('~q [fc=~d/F] for ~p', [ATrie, FalseCount, Goal])
+        ;   format('~q [fc=~d] for ~p', [ATrie, FalseCount, Goal])
+        )
+    ;   format('~q for ~p', [ATrie, Goal])
+    ).
 user:portray(Cont) :-
     compound(Cont),
     compound_name_arguments(Cont, '$cont$', [Clause, PC | Args]),
@@ -1635,10 +1644,10 @@ mon_propagate(Action, Head, ClauseRef) :-
     !,
     setup_call_cleanup(
         '$tbl_propagate_start'(Old),
-        propagate_assert(Head),
+        propagate_assert(Head),                 % eager monotonic dependencies
         '$tbl_propagate_end'(Old)),
     forall(dyn_affected(Head, ATrie),
-           '$mono_idg_changed'(ATrie, ClauseRef)).
+           '$mono_idg_changed'(ATrie, ClauseRef)). % lazy monotonic dependencies
 mon_propagate(retract, Head, _) :-
     !,
     mon_invalidate_dependents(Head).
@@ -1839,14 +1848,11 @@ try_reeval(ATrie, Goal, Return) :-
 try_reeval(ATrie, Goal, Return) :-
     tdebug(reeval, 'Planning reeval for ~p', [ATrie]),
     findall(Path, false_path(ATrie, Path), Paths0),
-    sort(0, @>, Paths0, Paths),
-    split_paths(Paths, Dynamic, Complete),
-    tdebug(forall('$member'(Path, Dynamic),
-                  tdebug(reeval, '  Re-eval dynamic path: ~p', [Path]))),
-    tdebug(forall('$member'(Path, Complete),
+    sort(0, @>, Paths0, Paths1),
+    clean_paths(Paths1, Paths),
+    tdebug(forall('$member'(Path, Paths),
                   tdebug(reeval, '  Re-eval complete path: ~p', [Path]))),
-    reeval_paths(Dynamic, ATrie),
-    reeval_paths(Complete, ATrie),
+    reeval_paths(Paths, ATrie),
     do_reeval(ATrie, Goal, Return).
 
 do_reeval(ATrie, Goal, Return) :-
@@ -1862,9 +1868,29 @@ do_reeval(ATrie, Goal, Return) :-
     ).
 
 
-split_paths([], [], []).
-split_paths([[_|Path]|T], DT, [Path|CT]) :-
-    split_paths(T, DT, CT).
+%!  clean_paths(+PathsIn, -Paths)
+%
+%   Clean the reevaluation paths. Get rid of   the head term for ranking
+%   and remove duplicate paths.
+
+clean_paths([], []).
+clean_paths([[_|Path]|T0], [Path|T]) :-
+    clean_paths(T0, Path, T).
+
+clean_paths([], _, []).
+clean_paths([[_|Path]|T0], CPath, T) :-
+    Path =@= CPath,
+    !,
+    clean_paths(T0, CPath, T).
+clean_paths([[_|Path]|T0], CPath, [Path|T]) :-
+    clean_paths(T0, CPath, T).
+
+%!  reeval_paths(+Paths, +Atrie)
+%
+%   Make Atrie valid again by re-evaluating nodes   in Paths. We stop as
+%   soon as Atrie  is  valid  again.  Note   that  we  may  not  need to
+%   reevaluate all paths because evaluating the   head  of some path may
+%   include other nodes in an SCC, making them valid as well.
 
 reeval_paths([], _) :-
     !.
@@ -1958,31 +1984,40 @@ reeval_node(ATrie) :-
 reeval_node(ATrie) :-
     '$mono_reeval_prepare'(ATrie, Size),
     !,
+    setup_call_cleanup(
+        '$tbl_propagate_start'(Old),
+        reeval_monotonic_node(ATrie, Size),
+        '$tbl_propagate_end'(Old)).
+reeval_node(_).
+
+reeval_monotonic_node(ATrie, Size) :-
     tdebug(reeval, 'Re-evaluating lazy monotonic ~p', [ATrie]),
-    (   '$idg_mono_affects_lazy'(ATrie, SrcTrie, Dep, Answers),
+    (   '$idg_mono_affects_lazy'(ATrie, _0SrcTrie, Dep, DepRef, Answers),
         (   Dep = dependency(Head, Cont, Skel)
         ->  (   '$member'(ClauseRef, Answers),
                 '$clause'(Head, _Body, ClauseRef, _Bindings),
                 tdebug(monotonic, 'Propagating ~p from ~p to ~p',
-                       [Head, SrcTrie, ATrie]),
+                       [Head, _0SrcTrie, ATrie]),
                 pdelim(Cont, Skel, ATrie),
                 fail
-            ;   '$idg_mono_empty_queue'(SrcTrie, ATrie)
+            ;   '$idg_mono_empty_queue'(DepRef)
             )
         ;   Dep = dependency(SrcSkel, true, Cont, Skel)
         ->  (   '$member'(Node, Answers),
                 '$tbl_node_answer'(Node, SrcSkel),
                 tdebug(monotonic, 'Propagating ~p from ~p to ~p',
-                       [Skel, SrcTrie, ATrie]),
+                       [Skel, _0SrcTrie, ATrie]),
                 pdelim(Cont, Skel, ATrie),
                 fail
-            ;   '$idg_mono_empty_queue'(SrcTrie, ATrie)
+            ;   '$idg_mono_empty_queue'(DepRef)
             )
+        ;   tdebug(monotonic, 'Skipped queued ~p, answers ~p',
+                   [Dep, Answers])
         ),
         fail
     ;   '$mono_reeval_done'(ATrie, Size)
     ).
-reeval_node(_).
+
 
 		 /*******************************
 		 *      EXPAND DIRECTIVES	*
