@@ -53,8 +53,8 @@
 	     delete_breakpoint/1,
 	     manpce/1,
 	     prolog_ide/1,
+	     rational/1,
 	     spypce/1,
-             trace/1,
 	     tracepce/1,
 	     atomic_list_concat/2,
 	     breakpoint_property/2,
@@ -65,21 +65,28 @@
 	     set_stream/2,
 	     source_file_property/2,
 	     string_codes/2,
+	     term_string/2,
 	     term_to_atom/2,
 	     absolute_file_name/3,
 	     atomic_list_concat/3,
 	     between/3,
 	     compound_name_arity/3,
 	     default/3,
+	     file_autoload_directives/3,
 	     file_name_extension/3,
 	     get_dict/3,
 	     nb_setarg/3,
+             pi_head/2,
+	     rational/3,
 	     send_list/3,
 	     setup_call_cleanup/3,
+	     stream_position_data/3,
 	     strip_module/3,
 	     xref_prolog_flag/4,
+	     get_dict/5,
 	     sub_string/5
 	   ]).
+:- autoload(library(pldoc/doc_process), [comment_modes/2]).
 
 resource(mode_pl_icon, image, image('32x32/doc_pl.xpm')).
 resource(breakpoint,   image, image('16x16/stop.xpm')).
@@ -112,6 +119,11 @@ resource(breakpoint,   image, image('16x16/stop.xpm')).
           view_threads                 = button(prolog),
           view_debug_messages          = button(prolog),
           -                            = button(prolog),
+          make_module                  = key('\\C-c\\C-o') + button(prolog),
+          export	               = key('\\C-c\\C-e') + button(prolog),
+          check_dependencies           = button(prolog),
+          update_dependencies          = key('\\C-c\\C-d') + button(prolog),
+          -                            = button(prolog),
           check_clause                 = key('\\C-c\\C-s') + button(prolog),
           setup_auto_indent            = button(prolog),
           insert_full_stop             = key(.),
@@ -172,6 +184,13 @@ class_variable(show_syntax_errors, {never,typing,pause},
                typing).
 class_variable(auto_colourise_size_limit, int, 100000,
                "Auto-colourise if buffer is smaller then this").
+class_variable(dependency_directive,
+               { 'autoload/1',
+                 'autoload/2',
+                 'use_module/1',
+                 'use_module/2'
+               }, 'autoload/2',
+               "How to insert new dependencies").
 
 variable(varmark_style,    style*,       get, "How to mark variables").
 variable(has_var_marks,    bool := @off, get, "Optimise a bit").
@@ -187,6 +206,14 @@ variable(body_indentation, int,          both, "Indentation for body-goals").
 variable(cond_indentation, int,          get, "Indent step for conditional").
 variable(quasiquotation_syntax,
                            name*,        both, "Default quasiquotation syntax").
+variable(dependency_directive,
+         { 'autoload/1',
+           'autoload/2',
+           'use_module/1',
+           'use_module/2'
+         },
+         both,
+         "How to insert new dependencies").
 
 class_variable(quasiquotation_syntax, name*, @nil).
 class_variable(body_indentation,      int,   4).
@@ -194,7 +221,6 @@ class_variable(cond_indentation,      int,   4).
 class_variable(indent_tabs,           bool,  @off,
                "Use tabs for indentation").
 
-
 icon(_, I:image) :<-
     "Return icon for mode"::
     catch(new(I, image(resource(mode_pl_icon))), _, fail).
@@ -252,7 +278,7 @@ expand_path(Term, D) :-
 :- pce_group(indent).
 
 :- pce_global(@prolog_neck_regex,
-              new(regex(':-|:->|:<-|-->'))).
+              new(regex(':-|:->|:<-|-->|=>|\\?=>'))).
 :- pce_global(@prolog_full_stop,
               new(regex('[^-#$&*+./:<=>?@\\\\^`~]\\.($|\\s)'))).
 :- pce_global(@prolog_decl_regex,
@@ -893,6 +919,334 @@ consult_selection(M) :->
     send(M, consult_region, From, To).
 
 
+		 /*******************************
+		 *         DEPENDENCIES		*
+		 *******************************/
+
+make_module(E) :->
+    "Turn the current file into a module file"::
+    (   get(E, prolog_module, Module)
+    ->  send(E, report, status, 'File defines Prolog module "%s"', Module)
+    ;   get(E, text_buffer, TB),
+        get(TB, file, File),
+        File \== @nil
+    ->  get(File, base_name, BaseName),
+        file_name_extension(Module, _, BaseName),
+        (   send(E, looking_at, '(%[^%!]|/\\*)', 0)
+        ->  get(TB, skip_comment, 0, skip_layout := @on, Where)
+        ;   Where = 0
+        ),
+        add_new_dependencies(
+            E,
+            [ :- module(Module, []) ],
+            Where)
+    ;   send(E, report, warning, 'No file')
+    ).
+
+export(E) :->
+    "Export the current predicate"::
+    (   get(E, prolog_term, Clause),
+        clause_pi(Clause, PI)
+    ->  send(E, do_export, PI)
+    ;   send(E, report, warning,
+             'Could not find predicate to export from here')
+    ).
+
+clause_pi(Var, _) :-
+    var(Var),
+    !,
+    fail.
+clause_pi((Head :- _), PI) :-
+    callable(Head),
+    Head \= (_:_),
+    pi_head(PI, Head).
+clause_pi((Head --> _), Name//Arity) :-
+    callable(Head),
+    Head \= (_:_),
+    pi_head(Name/Arity, Head).
+clause_pi(:-(_), _) :-
+    !,
+    fail.
+clause_pi(Head, PI) :-
+    callable(Head),
+    Head \= (_:_),
+    pi_head(PI, Head).
+
+do_export(E, PI:prolog) :->
+    "Add PI to the export list of the module"::
+    send(E, mark),
+    get(E, text_buffer, TB),
+    pi_head(PI, Head),
+    (   xref_comment(TB, Head, _Summary, Comment),
+        comment_modes(Comment, Modes),
+        pi_mode_head(PI, MHead),
+        member(ModeAndDet, Modes),
+        strip_det(ModeAndDet, MHead),
+        MHead =.. [_|Args],
+        Head2 =.. ['DUMMY'|Args],
+        term_string(Head2, Syn0,
+                    [ module(pldoc_modes),
+                      quoted(false)
+                    ]),
+        sub_string(Syn0, 6, _, 1, Syn)
+    ->  true
+    ;   true
+    ),
+    term_string(PI, PIs, [priority(999)]),
+    insert_export_after(E, PI, IPos0, How),
+    (   send(E, looking_at, '\\[\\]', IPos0)
+    ->  IPos is IPos0+2,
+        send(E, caret, IPos0+1),
+        send(E, insert, ' '),
+        send(E, newline_and_indent),
+        send(E, previous_line),
+        send(E, caret, IPos)
+    ;   How == empty
+    ->  send(E, caret, IPos0+1),
+        send(E, insert, ' ')
+    ;   send(E, caret, IPos0),
+        send(E, insert, ','),
+        (   send(E, looking_at, '\\s*%')
+        ->  send(E, insert_line_comment)
+        ;   true
+        ),
+        get(TB, skip_comment, IPos0+2, skip_layout := @on, IPos),
+        send(E, caret, IPos),
+        send(E, newline_and_indent),
+        send(E, previous_line),
+        send(E, indent_line)
+    ),
+    send(E, insert, PIs),
+    (   nonvar(Syn)
+    ->  send(E, insert_line_comment),
+        send(E, insert, Syn)
+    ;   true
+    ),
+    send(E, exchange_point_and_mark),
+    send(E, mark_status, inactive),
+    send(E, auto_colourise_buffer).
+
+strip_det(Mode is _, Mode) :- !.
+strip_det(//(Mode), Mode) :- !.
+strip_det(Mode, Mode).
+
+pi_mode_head(Name/Arity, Head) :-
+    functor(Head, Name, Arity).
+pi_mode_head(Name//Arity, Head) :-
+    functor(Head, Name, Arity).
+
+
+%!  insert_export_after(+Editor, +PI, -InsertPos, -How)
+%
+%   Figure out the current export list and   find  a place to insert the
+%   PI. Currently always inserts at the  end. Future versions may insert
+%   alphabetically or in file-order. Ideally we   should  figure out the
+%   current ordering convention and maintain that.
+
+insert_export_after(E, _PI, InsertPos, How) :-
+    between(1,2,_),
+    (   get(E, prolog_term, 0, @off, Pos, ModuleTerm),
+        ModuleTerm = (:- module(_,_List))
+    ;   send(E, make_module),
+        fail
+    ),
+    !,
+    Pos  = term_position(_,_,_,_,[MPos]),
+    MPos = term_position(_,_,_,_,[_,LPos]),
+    (   LPos = F-_T
+    ->  InsertPos = F,
+        How = empty
+    ;   LPos = list_position(_,_,ElemPos,none)
+    ->  last(ElemPos, EPos),			% for now always at the end
+        arg(2, EPos, InsertPos),
+        How = after_last
+    ).
+
+%member(X, Y, [X|_], [Y|_]).
+%member(X, Y, [_|Xs], [_|Ys]) :-
+%    member(X, Y, Xs, Ys).
+
+
+:- meta_predicate
+    to_kill_buffer(0).
+
+check_dependencies(E) :->
+    "See whether any dependencies are missing"::
+    get(E, text_buffer, TB),
+    file_autoload_directives(TB, Directives, [missing(true)]),
+    count_dependencies(Directives, Count),
+    (   Directives == []
+    ->  send(E, report, status, 'No missing dependencies')
+    ;   to_kill_buffer(maplist(portray_clause, Directives)),
+        send(E, report, warning,
+             'WARNING: %d missing dependencies.  \c
+              Use ^Y to insert required directives at caret.',
+             Count)
+    ).
+
+insert_dependencies(E) :->
+    "Insert :- autoload/2 directives"::
+    (   get(E, mark_status, active)
+    ->  get(E, selection, point(From, To))
+    ;   From = 0,
+        To = 0
+    ),
+    update_dependencies(E, From, To).
+
+update_dependencies(E) :->
+    "Update existing dependencies or add new block"::
+    get(E, find_dependencies, point(From, To)),
+    update_dependencies(E, From, To).
+
+update_dependencies(E, From, To) :-
+    read_terms_in_range(E, From, To, Terms),
+    directive_options(E, Terms, DirOptions),
+    get(E, text_buffer, TB),
+    file_autoload_directives(TB, Directives, [update(Terms)|DirOptions]),
+    (   append(Terms, New, Directives)
+    ->  (   New == []
+        ->  send(E, report, status, 'No updates required')
+        ;   add_new_dependencies(E, New, To),
+            count_dependencies(New, Count),
+            send(E, report, status, 'Added %d new dependencies', Count)
+        )
+    ;   send(E, delete, From, To),
+        add_new_dependencies(E, Directives, From),
+        send(E, report, status, 'Updated dependencies')
+    ).
+
+add_new_dependencies(E, Directives, At) :-
+    send(@emacs, location_history),
+    send(E, caret, At),
+    send(@emacs, location_history, title := 'Update dependencies'),
+    setup_call_cleanup(
+        pce_open(E, update, Out),
+        ( seek(Out, At, bof, _),
+          with_output_to(Out, maplist(portray_clause, Directives))
+        ),
+        close(Out)),
+    get(E, caret, End),
+    (   send(E, looking_at, '\\s*$', End)
+    ->  true
+    ;   send(E, insert, '\n')
+    ),
+    send(E, selection, At, End, highlight),
+    send(E, auto_colourise_buffer).
+
+directive_options(_E, Terms, []) :-
+    member(:-(Directive), Terms),
+    is_dependency(Directive),
+    !.
+directive_options(E, _, [directive(Pred)]) :-
+    get(E, dependency_directive, Atom),
+    term_string(Pred, Atom).
+
+count_dependencies(Directives, Deps) :-
+    maplist(directive_deps, Directives, Counts),
+    sum_list(Counts, Deps).
+
+directive_deps((:- autoload(_, Preds)), Count) :-
+    !,
+    length(Preds, Count).
+directive_deps((:- use_module(_, Preds)), Count) :-
+    !,
+    length(Preds, Count).
+directive_deps(_, 1).
+
+to_kill_buffer(Goal) :-
+    new(E, editor),
+    setup_call_cleanup(
+        pce_open(E, write, Out),
+        with_output_to(Out, Goal),
+        close(Out)),
+    get(E?text_buffer, size, Size),
+    send(E, grab, 0, Size),
+    free(E).
+
+read_selection(E, Terms:prolog) :<-
+    "Read the selection as a list of terms"::
+    get(E, selection, point(From, To)),
+    read_terms_in_range(E, From, To, Terms).
+
+read_terms_in_range(E, From, To, Terms) :-
+    To > From,
+    !,
+    setup_call_cleanup(
+        pce_open(E, read, In),
+        ( seek(In, From, bof, _),
+          read_terms_to(In, To, Terms)
+        ),
+        close(In)).
+read_terms_in_range(_, _, _, []).
+
+read_terms_to(In, End, Terms) :-
+    quiet_read_next(In, Term, _, Here),
+    !,
+    (   Term == end_of_file
+    ->  Terms = []
+    ;   (   Here > End
+        ->  Terms = []
+        ;   Terms = [Term|Rest],
+            read_terms_to(In, End, Rest)
+        )
+    ).
+read_terms_to(_, _, []).
+
+find_dependencies(E, Range:point) :<-
+    "Get start and end of dependencies"::
+    setup_call_cleanup(
+        pce_open(E, read, In),
+        find_dependencies(In, #{}, Dict),
+        close(In)),
+    get(E, text_buffer, TB),
+    (   _{start:Start, end:End0} :< Dict
+    ->  get(TB, scan, End0, line, 1, start, End),
+        new(Range, point(Start, End))
+    ;   _{module_decl_end:Start0} :< Dict
+    ->  get(TB, scan, Start0, line, 1, start, Start),
+        new(Range, point(Start, Start))
+    ;   _{program_start:Start0} :< Dict
+    ->  get(TB, scan, Start0, line, -1, start, Start),
+        new(Range, point(Start, Start))
+    ).
+
+find_dependencies(In, State0, State) :-
+    quiet_read_next(In, Term, F, T),
+    (   Term = :-(Directive)
+    ->  update_dep_state(Directive, F, T, State0, State1),
+        find_dependencies(In, State1, State)
+    ;   State = State0.put(program_start, F)
+    ).
+
+update_dep_state(module(_,_), _, T, State0, State) :-
+    !,
+    State = State0.put(module_decl_end, T).
+update_dep_state(Decl, S, E, State0, State) :-
+    is_dependency(Decl),
+    !,
+    (   get_dict(end, State0, _, State, E)
+    ->  true
+    ;   State = State0.put(#{start:S, end:E})
+    ).
+update_dep_state(_, _, _, State, State).
+
+is_dependency(autoload(_)).
+is_dependency(use_module(_)).
+is_dependency(autoload(_,_)).
+is_dependency(use_module(_,_)).
+
+quiet_read_next(In, Term, From, To) :-
+    between(1, 10, _),
+    read_term(In, Term,
+              [ syntax_errors(quiet),
+                term_position(Pos)
+              ]),
+    nonvar(Term),
+    stream_position_data(char_count, Pos, From),
+    character_count(In, To),
+    !.
+
+
                  /*******************************
                  *             PCE              *
                  *******************************/
