@@ -52,7 +52,9 @@ test_monotonic_lazy :-
                 tabling_monotonic_lazy_4,
                 tabling_monotonic_lazy_5,
                 tabling_monotonic_lazy_6,
-                tabling_monotonic_lazy_7
+                tabling_monotonic_lazy_7,
+                tabling_monotonic_lazy_8,
+                tabling_monotonic_lazy_9
               ]).
 
 :- meta_predicate
@@ -62,6 +64,8 @@ test_monotonic_lazy :-
     expect(?, 0, +),
     expect_valid(0),
     expect_invalid(0),
+    expect_forced(0),
+    incr_is_forced(0),
     expect_queued_answers(:, +).
 
 % ================================================================
@@ -269,6 +273,54 @@ test(lazy_reeval) :-
 :- end_tests(tabling_monotonic_lazy_7).
 
 
+:- begin_tests(tabling_monotonic_lazy_8).
+
+% Verify that a switch to incremental due to a retract propagates.
+% Note that eventually this may change.
+
+:- dynamic dm/1 as monotonic.
+:- dynamic xm/1 as monotonic.
+:- table aml/1 as (monotonic,lazy).
+:- table bml/1 as (monotonic,lazy).
+
+bml(X) :- aml(X) ; xm(X).
+aml(X) :- dm(X).
+
+test(indirect_lazy_retract) :-
+    cleanup([dm/1, xm/1]),
+    assert(dm(1)),
+    assert(dm(2)),
+    expect(X, bml(X), [1,2]),
+    assert(xm(3)),
+    assert(dm(4)),
+    retract(dm(1)),
+    expect_invalid(aml(_)),
+    expect_invalid(bml(_)),
+    expect_forced(aml(_)),
+    expect_forced(bml(_)),
+    expect(X, bml(X), [2,3,4]).
+
+:- end_tests(tabling_monotonic_lazy_8).
+
+:- begin_tests(tabling_monotonic_lazy_9).
+
+:- dynamic d/1 as monotonic.
+:- table p/1 as (monotonic,lazy).
+
+p(c) :-
+    d(a),
+    d(b).
+
+test(capture_new_dependencies) :-
+    cleanup([d/1]),
+    expect(X, p(X), []),
+    assert(d(a)),
+    expect(X, p(X), []),                % evaluation must create dependency on d(b).
+    assert(d(b)),
+    expect(X, p(X), [c]).
+
+:- end_tests(tabling_monotonic_lazy_9).
+
 		 /*******************************
 		 *         TEST HELPERS		*
 		 *******************************/
@@ -296,9 +348,16 @@ expect_valid(Goal) :-
 expect_invalid(Goal) :-
     assertion(incr_is_invalid(Goal)).
 
+expect_forced(Goal) :-
+    assertion(incr_is_forced(Goal)).
+
+incr_is_forced(Goal) :-
+    get_call(Goal, ATrie, _Templ),
+    '$idg_forced'(ATrie).
+
 queued_answers(To, From, Count) :-
     get_calls(To, DstTrie, _Ret),
-    '$idg_mono_affects_lazy'(DstTrie, SrcTrie, _Dep, Answers),
+    '$idg_mono_affects_lazy'(DstTrie, SrcTrie, _Dep, _DepRef, Answers),
     '$tbl_table_status'(SrcTrie, _Status, From0, _Skeleton),
     unqualify(To, From0, From),
     length(Answers, Count).
